// 基础变量配置
var gulp = require('gulp');
var plugins = require('gulp-load-plugins')({ pattern: '*' });
var reload = plugins.browserSync.reload;
var path = require('path');

var config = {
    path: {
        ejs: './src/pages/*.ejs',
        less: './src/less/*.less',
        img: './src/img/*.*',
        icon:'./src/img/icon/*.svg',
        js: './src/js/*.js'
    },
    dist: {
        html: './dist/html/',
        css: './dist/css/',
        img: './dist/img/',
        iconfont: './dist/fonts',
        js: './dist/js/'
    },
    autoprefixerBrowsers: [
        'ie >= 9',
        'ie_mob >= 10',
        'ff >= 30',
        'chrome >= 34',
        'safari >= 7',
        'opera >= 23',
        'ios >= 7',
        'android >= 2.3',
        'bb >= 10'
    ]
};

/* 开发 */
// 输出CSS: 编译less,前缀添加
function compileLess () {
  return gulp.src(config.path.less)
  .pipe(plugins.plumber({
    errorHandler: function ( err ) {
      console.log(err);
      this.emit('end');
    }
  }))
  .pipe(plugins.less())
  .pipe(plugins.autoprefixer({ browsers: config.autoprefixerBrowsers }))
  .pipe(gulp.dest(config.dist.css))
};
function minifyCss () {
  return gulp.src(['./dist/css/*.css', '!./dist/css/*.min.css'])
  .pipe(plugins.minifyCss())
  .pipe(plugins.rename({
      suffix: '.min'
  })) // 压缩css
  .pipe(gulp.dest(config.dist.css))
  .pipe(reload({stream:true}))
  .pipe(plugins.notify({
    message : 'css压缩编译输完成'
  })) // 输出压缩后的css
}
gulp.task('dev-css', gulp.series(compileLess, minifyCss));

// 输出HTML: 编译ejs
function compileEjs () {
  return gulp.src(config.path.ejs)
  .pipe(plugins.plumber({
    errorHandler: function ( err ) {
      console.log(err);
      this.emit('end');
    }
  }))
  .pipe(plugins.watch(config.path.ejs, { ignoreInitial: false }))
  .pipe(plugins.ejs({}, {ext: '.html'}))
  .pipe(gulp.dest(config.dist.html))
  .pipe(reload({stream:true}))
  .pipe(plugins.notify({
    message : 'ejs编译输出html完成'
  }));
}

// 输出js:
function js () {
  return gulp.src(config.path.js)
  .pipe(gulp.dest(config.dist.js))
}
function uglifyJs () {
  return gulp.src(['./dist/js/*.js', '!./dist/js/*.min.js'])
  .pipe(plugins.uglify())
  .pipe(plugins.rename({
      suffix: '.min'
  })) // 压缩JS
  .pipe(gulp.dest(config.dist.js))
  .pipe(reload({ stream: true }))
  .pipe(plugins.notify({
    message : 'js压缩输出完成'
  }));
}
gulp.task('dev-js', gulp.series(js, uglifyJs));

// 输出图片
function imagemin () {
  return gulp.src(config.path.img)
  .pipe(plugins.watch(config.path.img, { ignoreInitial: false }))
  .pipe(plugins.imagemin())
  .pipe(gulp.dest(config.dist.img))
  .pipe(reload({ stream : true }))
  .pipe(plugins.notify({
    message : '图片压缩完成'
  }));
}

// 生成字体图标
function generateIconFont () {
  return gulp.src(config.path.icon)
  .pipe(plugins.iconfontCss({
    fontName: 'iconFont',
    targetPath: '../../src/less/iconfont/iconfont.less',
    fontPath: '../fonts/'
  }))
  .pipe(plugins.iconfont({
    fontName: 'iconFont',
    formats: ['ttf', 'eot', 'woff', 'svg', 'woff2']
  }))
  .pipe(gulp.dest(config.dist.iconfont))
  .pipe(plugins.notify({
    message: 'svg图标生成完成'
  }));
}


// 自动刷新
function serve () {
  plugins.browserSync({
      server: {
          baseDir:'./'
      },
    startPath: config.dist.html
  });

  var watch_less = gulp.watch(config.path.less, gulp.series(compileLess, minifyCss));
  var watch_js = gulp.watch(config.path.js, gulp.series(js, uglifyJs));
  var watch_ejs = gulp.watch(config.path.ejs, gulp.parallel(compileEjs));
  var watch_img = gulp.watch(config.path.img, gulp.parallel(imagemin));
  var watch_icon = gulp.watch(config.path.icon, gulp.parallel(generateIconFont));

  watch_less.on('unlink', function ( route ) {
    var base = path.basename(route, '.less');
    var file = './dist/css/' + base + '.css';
    var minFile = './dist/css/' + base + '.min.css';
    plugins.del.sync([file, minFile]);
  });
  watch_js.on('unlink', function ( route ) {
    var base = path.basename(route);
    var file = './dist/js/' + base;
    var minFile = './dist/js/' + path.basename(route, '.js') + '.min.js';
    plugins.del.sync([file, minFile]);
  });
  watch_ejs.on('unlink', function ( route ) {
    var base = path.basename(route, 'ejs');
    var file = './dist/html/' + base + 'html';
    plugins.del.sync(file);
  });
  watch_img.on('unlink', function ( route ) {
    var base = path.basename(route);
    var file = './dist/img/' + base;
    plugins.del.sync(file);
  });
  watch_icon.on('unlink', function () {
    gulp.series(sprites);
  });
}

// 输出多余文件
function delFile () {
  return plugins.del([
      './dist/css/*.css','!./dist/css/*.min.css',
      './dist/js/*.js','!./dist/js/*.min.js',
      './src/less/*.css'
  ]);
}
gulp.task('del', function () {
  return plugins.del([
      './dist/css/*.css', './dist/css/*.orig','!./dist/css/*.min.css',
      './dist/js/*.js','!./dist/js/*.min.js',
      './src/less/*.css'
  ]);
});
gulp.task('del', delFile);

gulp.task('dev', gulp.parallel(
  gulp.series(js, uglifyJs),
  gulp.series(compileLess, minifyCss),
  compileEjs,
  generateIconFont,
  imagemin,
  delFile,
  serve
));
